/* ------------------------------------------------------------------------ */
/*                                                                          */
/*  Copyright (C)                                                           */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful, but     */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-    */
/*  ABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General        */
/*  Public License for more details.                                        */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   */
/*                                                                          */
/* ------------------------------------------------------------------------ */
/*  $Author: vogje02 $ $Revision: 1.7 $ $Date: 2002/07/28 17:13:20 $      */
/* ------------------------------------------------------------------------ */

#include "options.h"

/**
 * This class contains all nmap options
 */
COptions::COptions()
{
	int status = 0;

    WSADATA WSAData; 

    if( ( status = WSAStartup(MAKEWORD(1,1), &WSAData)) == 0 ) { 
		
		struct sockaddr_in localAddr;
		
		// Get local host name
		gethostname( hostName, 256 );

		// Get local host address
		struct hostent* hp = gethostbyname( hostName );
		memcpy( &(localAddr.sin_addr), hp->h_addr, hp->h_length); 

		// Set the host name field.
		sprintf( hostName, "%s", inet_ntoa( localAddr.sin_addr ) );
	
		WSACleanup();

    } else

		sprintf( hostName, "0.0.0.0" );

	
	// Scan mode
	scanMode = MODE_SYN_STEALTH;

	// Port range
	portRangeB = false;
	*portRangeS = '\0';

	// Device
	deviceB = false;
	*deviceS = '\0';

	// Source address (when used with device)
	sAddressB = false;
	*sAddressS = '\0';

	// Decoy
	decoyB = false;
	*decoyS = '\0';

	// Bounce
	bounceB = false;
	*bounceS = '\0';

	// Decoy
	portB = false;
	*portS = '\0';

	// Discovery
	discMode = DISC_TCP_ICMP;

	// Fragmentation
	fragB = false;

	// Ident
	identB = false;

	// Resolve all
	resolveAllB = false;

	// Do not resolve
	resolveNotB = false;

	// Fast scan
	fastScanB = false;

	// OSDetection
	osDetectionB = true;

	// Random host
	randomHostB = false;

	// Resume scan.
	resumeB = false;

    // Debug
	debugB = false;

	// Verbose
	verboseB = false;

	// Verbose
	veryVerboseB = false;

	// Throttle
	sprintf( throttle, "3" );

	// Host timeout
	hostTimeoutB = false;
	sprintf( hostTimeout, "-1" );

	// Min RTT
	minRttB = false;
	sprintf( minRtt, "3000" );

	// Max RTT
	maxRttB = false;
	sprintf( maxRtt, "6000" );
	
	// Initial RTT
	initialRttB = false;
	sprintf( initialRtt, "4500" );

	// Parallelism
	parallelB = false;
	sprintf( parallel, "-1" );

	// Delay
	delayB = false;
	sprintf( delay, "-1" );

	// Input file
	infileB = false;
	*infile = '\0';

	// Output file
	outfileB = false;
	*outfile = '\0';
	outfileType = OUTFILE_NORMAL;
	outfileAppend = false;

	// Windows specific options
	//

	// Windows get interfaces
	winInterfaceB = false;

	// No Pcap library
	winNoPcapB = false;

	// No HLP Api
	winNoHlpApiB = false;

	// No raw socket support
	winNoRawSocketB = false;

	// Force raw socket
	winForceRawSocketB = false;

	// NT4 routing
	winNt4RouteB = false;

	// Win Trace
	winTraceB = false;

	// Win service auto start
	autoStart = false;
}

/**
 * Create a options string, depending on the selected
 * options. This options string can be directly appended
 * to the nmap command.
 */
char* COptions::GetCommand( char* cmd )
{
	*cmd = '\0';

	if( winInterfaceB )

		strcat( cmd, "--win_list_interfaces" );
	
	else{

		// Scanmode
		switch( scanMode ) {
			case MODE_CONNECT: 
				strncat(cmd, "-sT ", 4);
				break;
			case MODE_SYN_STEALTH: 
				strncat(cmd, "-sS ", 4);
				break;
			case MODE_FIN_STEALTH: 
				strncat(cmd, "-sF ", 4);
				break;
			case MODE_PING_SWEEP: 
				strncat(cmd, "-sP ", 4);
				break;
			case MODE_UDP_SCAN: 
				strncat(cmd, "-sU ", 4);
				break;
			case MODE_NULL_SCAN: 
				strncat(cmd, "-sN ", 4);
				break;
			case MODE_XMAS_TREE: 
				strncat(cmd, "-sX ", 4);
				break;
			case MODE_IP_SCAN: 
				strncat(cmd, "-sO ", 4);
				break;
			case MODE_ACK_TREE: 
				strncat(cmd, "-sA ", 4);
				break;
			case MODE_WIN_SCAN: 
				strncat(cmd, "-sW ", 4);
				break;
			case MODE_RPC_SCAN: 
				strncat(cmd, "-sR ", 4);
				break;
			case MODE_LIST_SCAN: 
				strncat(cmd, "-sL ", 4);
		}

		// Discover mode
		switch( discMode ) {
			case DISC_TCP: 
				strncat(cmd, "-PT ", 4);
				break;
			case DISC_TCP_ICMP: 
				strncat(cmd, "-PT -PI ", 8);
				break;
			case DISC_ICMP: 
				strncat(cmd, "-PI ", 4);
				break;
			case DISC_NONE: 
				strncat(cmd, "-P0 ", 4);
				break;
		}

		// Port range
		if( portRangeB ) {   
			strncat( cmd, " -p ", 4 );
			strcat( cmd, portRangeS );
			strncat( cmd, " ", 1 );
		} 


		// Device
		if( deviceB ) {   
			strncat( cmd, " -e ", 4 );
			strcat( cmd, deviceS );
			strncat( cmd, " ", 1 );
		}

		// Source Address
		if( sAddressB ) {   
			strncat( cmd, " -S ", 4 );
			strcat( cmd, sAddressS );
			strncat( cmd, " ", 1 );
		}

		// Decoy
		if( decoyB ) {   
			strncat( cmd, " -D ", 4 );
			strcat( cmd, decoyS );
			strncat( cmd, " ", 1 );
		}

		// Bounce
		if( bounceB ) {
			strncat( cmd, " -b ", 4 );
			strcat( cmd, bounceS );
			strncat( cmd, " ", 1 );
		}

		// Source Port
		if( portB ) {   
			strncat( cmd, " -g ", 4 );
			strcat( cmd, portS );
			strncat( cmd, " ", 1 );
		}

		// General options
		if( fragB )
			strncat( cmd, "-f ", 4 );
		if( identB )
			strncat( cmd, "-I ", 4 );
		if( resolveAllB )
			strncat( cmd, "-R ", 4 );
		if( resolveNotB )
			strncat( cmd, "-n ", 4 );
		if( fastScanB )
			strncat(cmd, " -F ", 4);
		if( osDetectionB )
			strncat( cmd, "-O ", 4 );
		if( randomHostB )
			strncat( cmd, "-iR ", 4 );
		if( resumeB )
			strcat( cmd, "--resume " );

		// Debug options
		if( debugB )
			strncat( cmd, "-d ", 4 );
		if( verboseB )
			strncat( cmd, "-v ", 4 );
		if( veryVerboseB )
			strncat( cmd, "-vv ", 5 );

		// Throttle
		if( strstr( throttle, "0" ) )
			strncat(cmd, "-T 0 ", 5);
		else if( strstr( throttle, "1" ) )
			strncat(cmd, "-T 1 ", 5);
		else if( strstr( throttle, "2" ) )
			strncat(cmd, "-T 2 ", 5);
		else if( strstr( throttle, "3" ) )
			strncat(cmd, "-T 3 ", 5);
		else if( strstr( throttle, "4" ) )
			strncat(cmd, "-T 4 ", 5);
		else if( strstr( throttle, "5" ) )
			strncat(cmd, "-T 5 ", 5);

		if( hostTimeoutB )
			sprintf( cmd, "%s--host_timeout %s ", cmd, hostTimeout );

		if( minRttB )
			sprintf( cmd, "%s--min_rtt_timeout %s ", cmd, minRtt );

		if( maxRttB )
			sprintf( cmd, "%s--max_rtt_timeout %s ", cmd, maxRtt );

		if( initialRttB )
			sprintf( cmd, "%s--initial_rtt_timeout %s ", cmd, initialRtt );

		if( parallelB )
			sprintf( cmd, "%s--max_parallelism %s ", cmd, parallel );

		if( delayB )
			sprintf( cmd, "%s--scan_delay %s ", cmd, delay );

		// Input/output files
		if( infileB ) {
			strncat( cmd, "-iL \"", 5 );
			strcat( cmd, infile );
			strncat( cmd, "\" ", 2 );
		}

		if( outfileB ) {
			switch( outfileType )
			{
			case OUTFILE_NORMAL: 
				strncat(cmd, "-oN \"", 5);
				strcat( cmd, outfile );
				strncat( cmd, "\" ", 2 );
				break;
			case OUTFILE_XML: 
				strncat(cmd, "-oX \"", 5);
				strcat( cmd, outfile );
				strncat( cmd, "\" ", 2 );
				break;
			case OUTFILE_GREP: 
				strncat(cmd, "-oG \"", 5);
				strcat( cmd, outfile );
				strncat( cmd, "\" ", 2 );
				break;
			case OUTFILE_ALL: 
				strncat(cmd, "-oA \"", 5);
				strcat( cmd, outfile );
				strncat( cmd, "\" ", 2 );
				break;
			}
		}

		if( outfileAppend )
			strcat( cmd, "--append_output " );

		// No Pcap library
		if( winNoPcapB )
			strcat( cmd, "--win_nopcap " );

		if( winNoHlpApiB )
			strcat( cmd, "--win_noiphlpapi " );

		// No raw socket support
		if( winNoRawSocketB )
			strcat( cmd, "--win_norawsock " );

		// Force raw socket
		if( winForceRawSocketB )
			strcat( cmd, "--win_forcerawsock " );

		// NT4 routing
		if( winNt4RouteB )
			strcat( cmd, "--win_nt4route " );

		// Win Trace
		if( winTraceB )
			strcat( cmd, "--win_trace " );

		if( !infileB ) {
			strcat( cmd, hostName );
		}
	}

	strcat( cmd, "\0" );

	return cmd;
}

